package com.example.afrishop_v3.controllers;

import com.example.afrishop_v3.base.Result;
import com.example.afrishop_v3.enums.ResultCodeEnum;
import com.google.api.client.util.IOUtils;
import com.google.cloud.vision.v1.*;
import com.google.cloud.vision.v1.Feature.Type;
import com.google.protobuf.ByteString;
import com.google.protobuf.Descriptors;
import com.google.protobuf.GeneratedMessageV3;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cloud.gcp.vision.CloudVisionTemplate;
import org.springframework.core.io.ClassPathResource;
import org.springframework.core.io.ResourceLoader;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.client.RestTemplate;

import java.io.IOException;
import java.io.InputStream;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@RestController
@RequestMapping("search/image")
public class ImageSearchController extends Controller {
    // Imports the Google Cloud client library

    private final CloudVisionTemplate cloudVisionTemplate;


    private final ResourceLoader resourceLoader;

    public ImageSearchController(CloudVisionTemplate cloudVisionTemplate, ResourceLoader resourceLoader) {
        this.cloudVisionTemplate = cloudVisionTemplate;
        this.resourceLoader = resourceLoader;

//        try {
//            authExplicit();
//        } catch (IOException e) {
//            e.printStackTrace();
//        }
    }


    @GetMapping
    public Result main() throws Exception {
        //authExplicit();
        try (ImageAnnotatorClient vision = ImageAnnotatorClient.create()) {

            // The path to the image file to annotate
            String fileName = "./resources/wakeupcat.jpg";

            // Reads the image file into memory
            Path path = Paths.get(fileName);

            RestTemplate restTemplate = new RestTemplate();
            String url = "http://img.championat.com/news/big/l/c/ujejn-runi_1439911080563855663.jpg";
            byte[] imageBytes = restTemplate.getForObject(url, byte[].class);
            ByteString imgBytes = ByteString.copyFrom(imageBytes);

            // Builds the image annotation request
            List<AnnotateImageRequest> requests = new ArrayList<>();
            Image img = Image.newBuilder().setContent(imgBytes).build();
            Feature feat = Feature.newBuilder().setType(Type.LABEL_DETECTION).build();
            AnnotateImageRequest request =
                    AnnotateImageRequest.newBuilder().addFeatures(feat).setImage(img).build();
            requests.add(request);

            // Performs label detection on the image file
            BatchAnnotateImagesResponse response = vision.batchAnnotateImages(requests);
            List<AnnotateImageResponse> responses = response.getResponsesList();
            List<Map<String,Object>> list = new ArrayList<>();

            for (AnnotateImageResponse res : responses) {
                if (res.hasError()) {
                    System.out.format("Error: %s%n", res.getError().getMessage());
                    return new Result(ResultCodeEnum.VALIDATE_ERROR.getCode());
                }

                for (EntityAnnotation annotation : res.getLabelAnnotationsList()) {
                    Map<String,Object> map = new LinkedHashMap<>();
                    annotation
                            .getAllFields()
                            .forEach((k, v) -> map.put(k.getJsonName(),v));
                    list.add(map);
                }

                return new Result<>(list);
            }
        }
        return new Result<>(new ArrayList<>());
    }

}
